==> oscars_mgmnt_clms-0.0.5.dev20240208+28-py3-none-any.whl: oscars_mgmnt_clms/file_object.py <==
"""
Created on 02 Jully 2019

@author: Tonneau Stephane
"""

import re
from typing import List, Dict
from oscars_mgmnt_clms.utils.storage import StorageInput


class FileInfo(object):
    '''class that store  basic file description'''

    def __init__(self, filename, file_size=None, file_type=None, file_title=None):
        """ Initialisator of the class

        :param filename: the basename of the file
        :type filename: a string
        :param file_size: the size of the file
        :type file_size: a number
        :param file_type: the mime type of the file
        :type file_type: a string
        :param file_title: a title for the file
        :type file_title: a string
        """
        self.name = filename.replace("\\", "/")
        self.size = file_size
        self.type = file_type
        self.title = file_title
        self.category = None
        self.bbox = None

    def __str__(self):
        """return the string representation of the object

        :returns: a string representation of the object
        :rtype: string
        """
        return str(self.__dict__)

    def __repr__(self):
        """return a representation of the object

        :returns: a representation of the object
        :rtype: string
        """
        return str(self.__dict__)


class ObjectInfo(object):
    '''class that store a full object description'''
    HMA = "hma"
    INSPIRE = "inspire"

    @classmethod
    def from_path(cls, storage: StorageInput, src_folder: str) -> 'ObjectInfo':
        """ return an ObjectInfo based on the content of the provided directory

        :param storage: storage medium (filesystem or S3)
        :param src_folder: product source folder
        :returns: the description of the object
        """
        res = ObjectInfo(storage, src_folder)

        for filename in storage.list_files_recursively(src_folder):
            res.add(filename)

        return res

    @classmethod
    def from_regexp(cls, storage: StorageInput, src_folder, regexp=None) -> Dict[str, 'ObjectInfo']:
        """ return an ObjectInfo based on the content of the provided directory

        :returns: the description of the object
        """
        if regexp is None:
            return {"global": cls.from_path(storage, src_folder)}

        res = {}

        for filename in storage.list_files_recursively(src_folder):
            match = re.search(regexp, filename)
            if match:
                if len(match.groups()) > 0:
                    key = match.group(1)
                else:
                    key = "global"
                if key not in res:
                    res[key] = ObjectInfo(storage, src_folder)
                res[key].add(filename)

        return res

    def __init__(self, storage: StorageInput, product_path=None):
        """ Initialisator of the class
        """
        self.product_path = product_path
        self.metadatas: List[FileInfo] = []
        self.previews: List[FileInfo] = []
        self.up = []
        self.related: List[FileInfo] = []
        self.datas: List[FileInfo] = []
        self.reference_metadata = {}
        self.storage: StorageInput = storage

    def __str__(self):
        """ return a representation of the object

        :returns: a representation of the object
        :rtype: string
        """
        return str(self.__dict__)

    def __repr__(self):
        """ return a representation of the object

        :returns: a representation of the object
        :rtype: string
        """
        return str(self.__dict__)

    def size(self):
        """ return the number of files contained in the object

        :returns: the number of files contained in the object
        :rtype: Integer
        """
        return len(self.datas)+len(self.related)+len(self.previews)+len(self.metadatas)

    def get_product_path(self):
        """ return the product path information

        :returns: the product path information
        :rtype: string
        """
        return self.product_path

    def add(self, filename):
        """ add a file in the file list that describes an object

        :param filename: the basename of the file
        :type filename: a string
        """
        if ".xml" in filename:
            mime_type = "text/xml"
            name_info = "XML file"
            meta_type = None
            with self.storage.open_file_lines(filename) as fp:
                for line in fp:
                    if "<gmd:MD_Metadata" in line:
                        mime_type = "application/vnd.iso.19139+xml"
                        name_info = "Inspire metadata"
                        meta_type = self.INSPIRE
                        break
                    elif "<gmi:MI_Metadata" in line:
                        mime_type = "application/vnd.iso.19139+xml"
                        name_info = "Inspire metadata"
                        meta_type = self.INSPIRE
                        break
                    elif "<opt:EarthObservation" in line:
                        mime_type = "application/gml+xml"
                        name_info = "HMA metadata"
                        meta_type = self.HMA
                        break

            file_info = FileInfo(
                filename,
                self.storage.get_filesize(filename),
                mime_type,
                name_info
            )
            self.metadatas.append(file_info)
            if meta_type is not None:
                self.reference_metadata[meta_type] = file_info
        elif re.search(r"((_ql)|(quicklook.*))\.tif", filename.lower()):
            self.previews.append(FileInfo(filename, self.storage.get_filesize(filename), "image/tiff"))
        elif re.search(r"quicklook.*\.png", filename.lower()):
            self.previews.append(FileInfo(filename, self.storage.get_filesize(filename), "image/png"))
        elif ".tif" in filename.lower():
            self.datas.append(FileInfo(filename, self.storage.get_filesize(filename), "image/tiff"))
        elif ".hdf5" in filename.lower():
            self.datas.append(FileInfo(filename, self.storage.get_filesize(filename), "application/x-hdf5", "HDF5 binary"))
        elif ".hdf" in filename.lower():
            self.datas.append(FileInfo(filename, self.storage.get_filesize(filename), "application/x-hdf"))
        elif re.search(r"((_PVI_.*)|(_PVI))\.jp2", filename):
            self.previews.append(FileInfo(filename, self.storage.get_filesize(filename), "image/jp2"))
        elif ".jp2" in filename.lower():
            self.datas.append(FileInfo(filename, self.storage.get_filesize(filename), "image/jp2"))
        elif ".zip" in filename.lower():
            self.datas.append(FileInfo(filename, self.storage.get_filesize(filename), "application/zip"))
        elif ".txt" in filename.lower():
            self.related.append(FileInfo(filename, self.storage.get_filesize(filename), "text/plain"))
        elif all(substr not in filename.lower() for substr in [".info", ".wld"]):
            self.related.append(FileInfo(filename, self.storage.get_filesize(filename), None))

    def _get_split_key(self, match, dictionary, storage: StorageInput):
        """ helper method to insert on new fileobject entry
        in a dictionary on the base of the match object.
        :param match: a matcher structure
        :type match: a matcher structure
        :param dictionary: a dictionary of fileobject
        :type dictionary: a dictionary of fileobject

        :returns: the key that should be used of inserting the object
        :rtype: a string
        """
        key = None
        if match:
            if len(match.groups()) > 0:
                key = match.group(1)
            else:
                key = "global"
            if key not in dictionary:
                dictionary[key] = ObjectInfo(storage)
                dictionary[key].product_path = self.product_path
                dictionary[key].up = self.up
                dictionary[key].metadatas = self.metadatas
                dictionary[key].previews = self.previews
                dictionary[key].reference_metadata = self.reference_metadata
        return key

    def split(self, pattern="^.*$"):
        """ split the object in several pieces according a pattern
        :param pattern: a pattern (regular expression with group matching)
        :type pattern: a string

        :returns: a list of object splitted according the provided regexp
        :rtype: a list
        """
        res = {}
        for flinfo in self.related:
            key = self._get_split_key(re.search(pattern, flinfo.name), res, self.storage)
            if key is not None:
                res[key].related.append(flinfo)
        for flinfo in self.datas:
            key = self._get_split_key(re.search(pattern, flinfo.name), res, self.storage)
            if key is not None:
                res[key].datas.append(flinfo)
        return res

    def get_metafileinfo(self):
        """return the file information of the metadatafile (HMA or inspire)
        associated to the product

        :returns: the information of the metadatafile associated to the product
        :rtype: a FileInfo striucture
        """
        if self.is_hma():
            return self.reference_metadata[self.HMA]
        if self.is_inspire():
            return self.reference_metadata[self.INSPIRE]
        return None

    def is_hma(self):
        """indicates if an HMA metadata file is provided for the object

        :returns: true if an HMA metadata file is provided for the object
        :rtype: a boolean
        """
        return self.HMA in self.reference_metadata

    def is_inspire(self):
        """indicates if an inspire metadata file is provided for the object

        :returns: true if an inspire metadata file is provided for the object
        :rtype: a boolean
        """
        return self.INSPIRE in self.reference_metadata

==> oscars_mgmnt_clms-0.1.8.dev20240702+100-py3-none-any.whl: oscars_mgmnt_clms/file_object.py <==
"""
Created on 02 Jully 2019

@author: Tonneau Stephane
"""

import re
from typing import List, Dict
from oscars_mgmnt_clms.utils.storage import StorageInput


class FileInfo(object):
    """class that store  basic file description"""

    def __init__(self, filename, file_size=None, file_type=None, file_title=None):
        """Initialisator of the class

        :param filename: the basename of the file
        :type filename: a string
        :param file_size: the size of the file
        :type file_size: a number
        :param file_type: the mime type of the file
        :type file_type: a string
        :param file_title: a title for the file
        :type file_title: a string
        """
        self.name = filename.replace("\\", "/")
        self.size = file_size
        self.type = file_type
        self.title = file_title
        self.category = None
        self.bbox = None

    def __str__(self):
        """return the string representation of the object

        :returns: a string representation of the object
        :rtype: string
        """
        return str(self.__dict__)

    def __repr__(self):
        """return a representation of the object

        :returns: a representation of the object
        :rtype: string
        """
        return str(self.__dict__)


class ObjectInfo(object):
    """class that store a full object description"""

    HMA = "hma"
    INSPIRE = "inspire"

    @classmethod
    def from_path(cls, storage: StorageInput, src_folder: str) -> "ObjectInfo":
        """return an ObjectInfo based on the content of the provided directory

        :param storage: storage medium (filesystem or S3)
        :param src_folder: product source folder
        :returns: the description of the object
        """
        res = ObjectInfo(storage, src_folder)

        for filename in storage.list_files_recursively(src_folder):
            res.add(filename)

        return res

    @classmethod
    def from_regexp(
        cls, storage: StorageInput, src_folder, regexp=None
    ) -> Dict[str, "ObjectInfo"]:
        """return an ObjectInfo based on the content of the provided directory

        :returns: the description of the object
        """
        if regexp is None:
            return {"global": cls.from_path(storage, src_folder)}

        res = {}

        for filename in storage.list_files_recursively(src_folder):
            match = re.search(regexp, filename)
            if match:
                if len(match.groups()) > 0:
                    key = match.group(1)
                else:
                    key = "global"
                if key not in res:
                    res[key] = ObjectInfo(storage, src_folder)
                res[key].add(filename)

        return res

    def __init__(self, storage: StorageInput, product_path=None):
        """Initialisator of the class"""
        self.product_path = product_path
        self.metadatas: List[FileInfo] = []
        self.previews: List[FileInfo] = []
        self.up = []
        self.related: List[FileInfo] = []
        self.datas: List[FileInfo] = []
        self.reference_metadata = {}
        self.storage: StorageInput = storage

    def __str__(self):
        """return a representation of the object

        :returns: a representation of the object
        :rtype: string
        """
        return str(self.__dict__)

    def __repr__(self):
        """return a representation of the object

        :returns: a representation of the object
        :rtype: string
        """
        return str(self.__dict__)

    def size(self):
        """return the number of files contained in the object

        :returns: the number of files contained in the object
        :rtype: Integer
        """
        return (
            len(self.datas)
            + len(self.related)
            + len(self.previews)
            + len(self.metadatas)
        )

    def get_product_path(self):
        """return the product path information

        :returns: the product path information
        :rtype: string
        """
        return self.product_path

    def add(self, filename):
        """add a file in the file list that describes an object

        :param filename: the basename of the file
        :type filename: a string
        """
        if ".xml" in filename:
            mime_type = "text/xml"
            name_info = "XML file"
            meta_type = None
            with self.storage.open_file_lines(filename) as fp:
                for line in fp:
                    if "<gmd:MD_Metadata" in line:
                        mime_type = "application/vnd.iso.19139+xml"
                        name_info = "Inspire metadata"
                        meta_type = self.INSPIRE
                        break
                    elif "<gmi:MI_Metadata" in line:
                        mime_type = "application/vnd.iso.19139+xml"
                        name_info = "Inspire metadata"
                        meta_type = self.INSPIRE
                        break
                    elif "<opt:EarthObservation" in line:
                        mime_type = "application/gml+xml"
                        name_info = "HMA metadata"
                        meta_type = self.HMA
                        break

            file_info = FileInfo(
                filename, self.storage.get_filesize(filename), mime_type, name_info
            )
            self.metadatas.append(file_info)
            if meta_type is not None:
                self.reference_metadata[meta_type] = file_info
        elif re.search(r"((_ql)|(quicklook.*))\.tif", filename.lower()):
            self.previews.append(
                FileInfo(filename, self.storage.get_filesize(filename), "image/tiff")
            )
        elif re.search(r"quicklook.*\.png", filename.lower()):
            self.previews.append(
                FileInfo(filename, self.storage.get_filesize(filename), "image/png")
            )
        elif ".tif" in filename.lower():
            self.datas.append(
                FileInfo(filename, self.storage.get_filesize(filename), "image/tiff")
            )
        elif ".hdf5" in filename.lower():
            self.datas.append(
                FileInfo(
                    filename,
                    self.storage.get_filesize(filename),
                    "application/x-hdf5",
                    "HDF5 binary",
                )
            )
        elif ".hdf" in filename.lower():
            self.datas.append(
                FileInfo(
                    filename, self.storage.get_filesize(filename), "application/x-hdf"
                )
            )
        elif re.search(r"((_PVI_.*)|(_PVI))\.jp2", filename):
            self.previews.append(
                FileInfo(filename, self.storage.get_filesize(filename), "image/jp2")
            )
        elif ".jp2" in filename.lower():
            self.datas.append(
                FileInfo(filename, self.storage.get_filesize(filename), "image/jp2")
            )
        elif ".zip" in filename.lower():
            self.datas.append(
                FileInfo(
                    filename, self.storage.get_filesize(filename), "application/zip"
                )
            )
        elif ".txt" in filename.lower():
            self.related.append(
                FileInfo(filename, self.storage.get_filesize(filename), "text/plain")
            )
        elif all(substr not in filename.lower() for substr in [".info", ".wld"]):
            self.related.append(
                FileInfo(filename, self.storage.get_filesize(filename), None)
            )

    def _get_split_key(self, match, dictionary, storage: StorageInput):
        """helper method to insert on new fileobject entry
        in a dictionary on the base of the match object.
        :param match: a matcher structure
        :type match: a matcher structure
        :param dictionary: a dictionary of fileobject
        :type dictionary: a dictionary of fileobject

        :returns: the key that should be used of inserting the object
        :rtype: a string
        """
        key = None
        if match:
            if len(match.groups()) > 0:
                key = match.group(1)
            else:
                key = "global"
            if key not in dictionary:
                dictionary[key] = ObjectInfo(storage)
                dictionary[key].product_path = self.product_path
                dictionary[key].up = self.up
                dictionary[key].metadatas = self.metadatas
                dictionary[key].previews = self.previews
                dictionary[key].reference_metadata = self.reference_metadata
        return key

    def split(self, pattern="^.*$"):
        """split the object in several pieces according a pattern
        :param pattern: a pattern (regular expression with group matching)
        :type pattern: a string

        :returns: a list of object splitted according the provided regexp
        :rtype: a list
        """
        res = {}
        for flinfo in self.related:
            key = self._get_split_key(
                re.search(pattern, flinfo.name), res, self.storage
            )
            if key is not None:
                res[key].related.append(flinfo)
        for flinfo in self.datas:
            key = self._get_split_key(
                re.search(pattern, flinfo.name), res, self.storage
            )
            if key is not None:
                res[key].datas.append(flinfo)
        return res

    def get_metafileinfo(self):
        """return the file information of the metadatafile (HMA or inspire)
        associated to the product

        :returns: the information of the metadatafile associated to the product
        :rtype: a FileInfo striucture
        """
        if self.is_hma():
            return self.reference_metadata[self.HMA]
        if self.is_inspire():
            return self.reference_metadata[self.INSPIRE]
        return None

    def is_hma(self):
        """indicates if an HMA metadata file is provided for the object

        :returns: true if an HMA metadata file is provided for the object
        :rtype: a boolean
        """
        return self.HMA in self.reference_metadata

    def is_inspire(self):
        """indicates if an inspire metadata file is provided for the object

        :returns: true if an inspire metadata file is provided for the object
        :rtype: a boolean
        """
        return self.INSPIRE in self.reference_metadata
